########################################################
# 1. Visualize clustering in two dimensions
########################################################
library(ggplot2)
draw.obs <- function(Y,K){

  X = as.data.frame(prcomp(Y)$x)
  
  mu_x = sapply(1:max(K),function(z){mean(X[which(K==z),1])})
  mu_y = sapply(1:max(K),function(z){mean(X[which(K==z),2])})
  
  ggplot(X[,1:2], aes(x=PC1, y=PC2)) +
    geom_point(color=K+1, shape=K, size=1, alpha = 0.5) +
    theme_void() +
    annotate("text", x=mu_x, y=mu_y, label=1:max(K), size=5, fontface = "bold")
}

########################################################
# 2. Generate bootstrap samples
########################################################

GenBsSamps <- function(X, nb, clustering, stack=T){
  # set.seed(123)
  n <- nrow(X)                      #n: number of observations
  bet <- array(0, dim = c(nb, n))   #nb: number of bootstrap samples
  Zb <- array(0, dim = c(nb,n))     
  
  Zb.stack <- NULL
  m <- rep(NA,nb)
  for (i in 1:nb){
    bet[i,] = sample(c(1:n),n,replace = T)	# pick random indices
    Xb = as.data.frame(X[bet[i,],])
    
    Zb[i,] = clustering(Xb, newdata=X)      # train with bootstraped sample
                                            # and predict using new cluster
    m[i] = length(unique(Zb[i,]))
    Zb.stack = c(Zb.stack, Zb[i,])
    print(i)
  }
  
  if(stack==T){
    return(list(Z=Zb.stack,m=m))
  } else {
    return(list(Z=Zb,m=m))
  }
}

########################################################
# 3. Align bootstrap samples
########################################################

align <- function(Zb.stack, nb, folder="align"){
  wd=getwd()
  setwd("OTA/package5")
  if(!(dir.exists(folder))) dir.create(folder)
  file.copy("labelsw_bunch", folder)
  setwd(folder)
  
  n = length(Zb.stack)/nb
  rfcls = rep(1,n)
  zbcls = c(rfcls,Zb.stack)-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  system(paste("./labelsw_bunch -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb+1," -2",sep=""))
  
  dist <- list()
  res <- list()
  wt <- list()
  smry <- list()
    
  for(i in 1:nb){
    nam = i
    
    dist[[i]] <- read.table(paste("zb_",nam,".par",sep="")) 
    res[[i]] <- read.table(paste("zb_",nam,".ls",sep="")) 
    wt[[i]] <- read.table(paste("zb_",nam,".wt",sep=""))
    smry[[i]] <- read.table(paste("zb_",nam,".summary",sep=""))  
  }
  
  setwd(wd)
  return(list(dist=dist,res=res,wt=wt,summary=smry))
}

########################################################
# 3.1 Align bootstrap samples - 2nd method
########################################################

align2 <- function(Zb.stack, nb, folder="align"){
  wd=getwd()
  setwd("OTA/package5")
  if(!(dir.exists(folder))) dir.create(folder)
  file.copy("labelsw_bunch2", folder)
  setwd(folder)
  
  n = length(Zb.stack)/nb
  rfcls = rep(1,n)
  zbcls = c(rfcls,Zb.stack)-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  idx = as.numeric(system(paste("./labelsw_bunch2 -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb+1," -2",sep=""), intern=T))
  
  setwd(wd)
  
  return(list(idx=idx))
  
}

########################################################
# 4. Find representative samples - quick version
########################################################

Repre.quick <- function(Zb.stack, dist, wt, method="median"){
  nb = length(dist)
  n = length(Zb.stack)/nb
  m = dist[[1]][-1,1]
  
  avg.dist = rep(NA,nb)
  cnt.nrst = rep(NA,nb)
  P.raw = list()
  
  for(i in 1:nb){
    avg.dist[i] = mean(dist[[i]][,2])
  }
  
  std = sqrt(var(avg.dist))
  
  for(i in 1:nb){
    P.raw[[i]] = matrix(0,n,m[i])
    for(k in 1:n){
      P.raw[[i]][k, Zb.stack[n*(i-1)+k]] = 1
    }
    cnt.nrst[i] = sum(dist[[i]][,2]<std)
  }  
  
  if(method=="median") idx = which.min(avg.dist)
  else if(method=="mode") idx = which.max(cnt.nrst)
  
  wttemp = as.matrix(wt[[idx]])
  K.rf = ncol(wttemp)
  
  P.tild = matrix(NA,n,K.rf*nb)
  P.tild.sum = matrix(0,n,K.rf)
  
  for(k in 1:nb){
    wtsub = wttemp[(sum(m[0:(k-1)])+1):sum(m[0:k]),]
    P.tild = P.raw[[k]]%*%(wtsub/rowSums(wtsub))
    P.tild.sum = P.tild.sum + P.tild
  }
  
  P.bar = P.tild.sum/nb
  P.bar.hrd.asgn = apply(P.bar,1,which.max)
  
  return(list(repre=P.bar.hrd.asgn,idx=idx))
}

########################################################
# 4.1 Find representative samples - 2nd version
########################################################

Repre2 <- function(Zb.stack, nb, idx, threshold=0.8, alpha=0.1){
  n = length(Zb.stack)/nb
  K.rf = max(Zb.stack[(n*(idx-1)+1):(n*idx)])
    
  wd=getwd()
  setwd("OTA/package5")
  
  rfcls = Zb.stack[(n*(idx-1)+1):(n*idx)]
  zbcls = c(rfcls,Zb.stack)-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  system(paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb+1," -t ",threshold," -a ",alpha," -2",sep="")
         ,ignore.stdout = T)
  
  dist <- read.table("zb.par")
  wt <- read.table("zb.wt")

  setwd(wd)
  
  m = dist[-1,1]
  
  P.raw = list()
  
  for(i in 1:nb){
    P.raw[[i]] = matrix(0,n,m[i])
    for(k in 1:n){
      P.raw[[i]][k, Zb.stack[n*(i-1)+k]] = 1
    }
  }  
    
  wt = as.matrix(wt)
  
  P.tild = matrix(NA,n,K.rf*nb)
  P.tild.sum = matrix(0,n,K.rf)
    
    for(k in 1:nb){
      wtsub = wt[(sum(m[0:(k-1)])+1):sum(m[0:k]),]
      P.tild = P.raw[[k]]%*%(wtsub/ifelse(rowSums(wtsub)==0,1,rowSums(wtsub)))
      P.tild.sum = P.tild.sum + P.tild
    }
    
  P.bar = P.tild.sum/nb
  P.bar.hrd.asgn = apply(P.bar,1,which.max)
  K.rf = max(P.bar.hrd.asgn)
  
  setwd("OTA/package5")
  
  rfcls = P.bar.hrd.asgn
  zbcls = c(rfcls,Zb.stack)-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  system(paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb+1," -t ",threshold," -a ",alpha," -2",sep=""))
  
  dist <- read.table("zb.par")
  res <- read.table("zb.ls") 
  wt <- read.table("zb.wt")
  smry <- read.table("zb.summary",nrows=K.rf)
  jaccard_mat <- read.table("zb.summary",skip=K.rf,nrows=K.rf)
  confset.idx <- list()
  confset <- list()
  for(i in 1:K.rf){
    confset.idx[[i]] <- as.integer(read.table("zb.summary",skip=(2*K.rf+i-1),nrows=1))[-c(1,2)]+1
    confset[[i]] = rep(0,n)
    confset[[i]][confset.idx[[i]]]=1
    
  }
  
  setwd(wd)
  
  return(list(repre=P.bar.hrd.asgn,
              dist=dist,res=res,wt=wt,
              summary=smry,
              jaccard_mat = jaccard_mat,
              confset=confset))
}


########################################################
# 5. Find representative samples - double run version
########################################################

Repre <- function(Zb.stack, dist, wt, folder="align2", method="median"){
  nb = length(dist)
  n = length(Zb.stack)/nb
  m = dist[[1]][-1,1]
  
  P.raw = list()
  
  for(i in 1:nb){
    P.raw[[i]] = matrix(0,n,m[i])
    for(k in 1:n){
      P.raw[[i]][k, Zb.stack[n*(i-1)+k]] = 1
    }
  }  
  
  P.bar.hrd.asgn = list()
  Zb.stack2 = NULL
  
  for(i in 1:nb){
    
    wttemp = as.matrix(wt[[i]])
    K.rf = ncol(wttemp)
    
    P.tild = matrix(NA,n,K.rf*nb)
    P.tild.sum = matrix(0,n,K.rf)
    
    for(k in 1:nb){
      wtsub = wttemp[(sum(m[0:(k-1)])+1):sum(m[0:k]),]
      P.tild = P.raw[[k]]%*%(wtsub/rowSums(wtsub))
      P.tild.sum = P.tild.sum + P.tild
    }
    
    P.bar = P.tild.sum/nb
    P.bar.hrd.asgn[[i]] = apply(P.bar,1,which.max)
    
    Zb.stack2 = c(Zb.stack2, P.bar.hrd.asgn[[i]])
  }
  
  # call align function #
  Zb.align <- align(Zb.stack2, nb, folder=folder)
  #######################
  
  dist2 = Zb.align$dist
  wt2 = Zb.align$wt
  
  avg.dist = rep(NA,nb)
  cnt.nrst = rep(NA,nb)
  P.raw = list()

  for(i in 1:nb){
    avg.dist[i] = mean(dist2[[i]][,2])
  }
  
  std = sqrt(var(avg.dist)/n)
  
  for(i in 1:nb){
    P.raw[[i]] = matrix(0,n,m[i])
    for(k in 1:n){
      P.raw[[i]][k, Zb.stack2[n*(i-1)+k]] = 1
    }
    cnt.nrst[i] = sum(dist2[[i]][,2]<std)
  }  
  
  if(method=="median") idx = which.min(avg.dist)
  else if(method=="mode") idx = which.max(cnt.nrst)
  
  return(list(repre=P.bar.hrd.asgn[[idx]],idx=idx))
}

########################################################
# 6. Statistics for stability of a result of clustering
########################################################

StatForClst <- function(Zb.repre, Zb.stack, nb=(length(Zb.stack)/length(Zb.repre)), t=0.8) {
  wd=getwd()
  setwd("OTA/package5")
  
  rfcls = Zb.repre
  zbcls = c(rfcls,Zb.stack)-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  system(paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb+1," -t ",t," -2",sep=""))
    
  dist <- read.table("zb.par")
  res <- read.table("zb.ls") 
  wt <- read.table("zb.wt")
  smry <- read.table("zb.summary")
  
  setwd(wd)

  # Calculate 5 statistics #
  K.rf = ncol(res)
  P = I = S = M = O = rep(0,K.rf)

  m = dist[,1]
  for(k in 1:nb){
    wttemp = round(as.matrix(wt[sum(m[0:(k-1)],1):sum(m[0:k]),]),5)
    ind = which(wttemp>0)
    rowind = ifelse(ind%%m[k]==0,m[k],ind%%m[k])
    for(j in 1:K.rf){
      colcnt = (1+m[k]*(j-1) <= ind)&(ind <=m[k]*j)
      if(sum(colcnt)==1){
        rowcnt = (rowind == rowind[which(colcnt)])
        if(sum(rowcnt)==1) P[j] = P[j]+1
        else if(sum(rowcnt)>1) M[j] = M[j]+1
      } else if (sum(colcnt)>1){
        if(prod(wttemp[rowind[which(colcnt)],j] == rowSums(wttemp[rowind[which(colcnt)],]))){
          if(max(wttemp[ind[which(colcnt)]]/sum(wttemp[ind[which(colcnt)]]))>=t) I[j] = I[j] +1
          else S[j] = S[j]+1
        } else O[j] = O[j]+1
      }
    }
  }
  stat = rbind(P,I,S,M,O)
  colnames(stat) = colnames(wt)
  
  return(list(dist=dist,res=res,wt=wt,summary=smry,statistics=stat))
}

########################################################
# 7. Overall clustering stability
########################################################

Entropy <- function(dist){
  std = sqrt(var(dist))
  cate = round(dist/std)
  px  = table(cate)/length(cate)
  lpx = log(px, base=2)
  ent = -sum(px*lpx)
  return(ent)
}

########################################################
# 8. Confidence Set
########################################################

ConfSet <- function(res, Zb.stack, m, alpha=0.1, n=length(Zb.stack)/length(m), nb=length(m), j=1:ncol(res) ){
  wd=getwd()
  setwd("OTA/package5")
  
  confset = list()
  cfset.stat = list()
  
  m.cum = apply(as.array(1:nb),1,function(x) sum(m[1:x]))
  
  j = j[-which(apply(res,2,function(x) prod(x==-1))==1)]
  for(k in j){
    idx.cls.mtc = which(res[,k]!=-1 & res[,k]<2)
    #val.cls.mtc = res[which(res[,k]!=-1 & res[,k]<2),k]
    
    idx.bt = apply(as.array(idx.cls.mtc),1,function(x) which(m.cum>=x)[1])
    idx.cls.bt = m[idx.bt]-(m.cum[idx.bt]-idx.cls.mtc)
    
    out = list()
    for(kk in 1:length(idx.cls.bt)){
      pred = Zb.stack[((idx.bt[kk]-1)*n+1):(idx.bt[kk]*n)]
      id = which(pred == idx.cls.bt[kk])
      out[[kk]] = c(length(id),id)
    }
    
    file.create(paste("cluster",k,".txt",sep=""))
    lapply(out, write, paste("cluster",k,".txt",sep=""), append=TRUE, ncolumns=n)
    
    system(paste("./confset -i cluster",k,".txt -o confset",k,".txt -a ", alpha, sep=""))
    
    cfset.idx = as.matrix(read.table(paste("confset",k,".txt",sep=""), nrows=1))
    cfset.stat[[k]] = as.matrix(read.table(paste("confset",k,".txt",sep=""), skip=1, nrows=1, col.names=1:6))
    confset[[k]] = rep(0,n)
    confset[[k]][cfset.idx[-1]]=1
  }
  
  setwd(wd)
  return(list(confset=confset,cfset.stat=cfset.stat))
}

########################################################
# 9. Draw plots
########################################################

library(ggplot2)
library(dplyr)
draw.obs2 <- function(X,true_cell_cls){
  X = as.data.frame(prcomp(X)$x[,1:2])
  X = as.data.frame(X)
  K = true_cell_cls
  A = as.character(true_cell_cls)
  
  s = cbind(X,A) %>% split(K)
  ch = s %>% lapply(., function(el) chull(el$PC1, el$PC2))
  ch = lapply(names(ch), function(x) s[[x]][ch[[x]],]) %>% do.call(rbind, .) 
  
  mu_x = sapply(1:max(K),function(z){mean(X[which(K==z),1])})
  mu_y = sapply(1:max(K),function(z){mean(X[which(K==z),2])})
  
  ggplot(as.data.frame(X[,1:2]), aes(x=PC1, y=PC2, color = A)) +
    geom_point(shape=19, size=1, alpha=0.5) +
    geom_polygon(data = ch, aes(fill = A), alpha = 0.2) +
    #guides(fill=guide_legend(title="Cell Type"), color=guide_legend(title="Cell Type")) +
    theme_classic() +
    annotate("text", x=mu_x, y=mu_y, label=1:max(K), size=4, fontface = "bold")
}

draw.obs3 <- function(X,true_cell_cls){
  X = as.data.frame(prcomp(X)$x[,1:2])
  X = as.data.frame(X)
  K = true_cell_cls
  A = as.character(true_cell_cls)
  
  s = cbind(X,A) %>% split(K)
  ch = s %>% lapply(., function(el) chull(el$PC1, el$PC2))
  ch = lapply(names(ch), function(x) s[[x]][ch[[x]],]) %>% do.call(rbind, .) 
  
  #mu_x = sapply(1:max(K),function(z){mean(X[which(K==z),1])})
  #mu_y = sapply(1:max(K),function(z){mean(X[which(K==z),2])})
  
  ggplot(as.data.frame(X[,1:2]), aes(x=PC1, y=PC2, color = A)) +
    geom_point(shape=19, size=1) +
    geom_polygon(data = ch[which(ch$A==1),], aes(fill = ch$A[which(ch$A==1)]), alpha = 0.2) +
    guides(fill=FALSE, color=FALSE) +
    scale_color_manual(values=c("black","red")) +
    scale_fill_manual(values=c("red")) +
    theme_classic()
  #annotate("text", x=mu_x, y=mu_y, label=1:max(K), size=4, fontface = "bold")
}

draw.obs4 <- function(X,true_cell_cls){
  X = as.data.frame(X)
  K = true_cell_cls
  A = as.character(true_cell_cls)
  
  s = cbind(X,A) %>% split(K)
  ch = s %>% lapply(., function(el) chull(el$PC1, el$PC2))
  ch = lapply(names(ch), function(x) s[[x]][ch[[x]],]) %>% do.call(rbind, .) 
  
  mu_x = sapply(unique(K),function(z){mean(X[which(K==z),1])})
  mu_y = sapply(unique(K),function(z){mean(X[which(K==z),2])})
  
  ggplot(as.data.frame(X[,1:2]), aes(x=PC1, y=PC2, color = A)) +
    geom_point(shape=19, size=1, alpha=0.5) +
    geom_polygon(data = ch, aes(fill = A), alpha = 0.2) +
    guides(color = guide_legend(title="Cluster"), fill=guide_legend(title="Cluster")) +
    theme_classic() +
    annotate("text", x=mu_x, y=mu_y, label=unique(K), size=2, fontface = "bold")
}

draw.obs5 <- function(Y,K,title="",xlab="",ylab=""){
  
  X = data.frame(PC1=Y[,1],PC2=Y[,2])
  
  mu_x = sapply(1:max(K),function(z){mean(X[which(K==z),1])})
  mu_y = sapply(1:max(K),function(z){mean(X[which(K==z),2])})
  
  #if(title==""){
  #  ggplot(X[,1:2], aes(x=PC1, y=PC2)) +
  #    geom_point(color=K+1, shape=K, size=1, alpha = 0.5) +
  #    theme_classic() +
  #    labs(x="",y="")  
  #} else{
  ggplot(X[,1:2], aes(x=PC1, y=PC2)) +
    geom_point(color=K+1, shape=K, size=1, alpha = 0.5) +
    theme_classic() +
    labs(x=xlab,y=ylab) +
    ggtitle(title) +
    theme(plot.title = element_text(face="bold", hjust = 0.5))
  #annotate("text", x=mu_x, y=mu_y, label=1:max(K), size=5, fontface = "bold")
  #}
}

draw.obs6 <- function(Y,K,title="",xlab="",ylab=""){
  
  X = data.frame(PC1=Y[,1],PC2=Y[,2])
  
  mu_x = sapply(1:max(K),function(z){mean(X[which(K==z),1])})
  mu_y = sapply(1:max(K),function(z){mean(X[which(K==z),2])})
  
  #if(title==""){
  #  ggplot(X[,1:2], aes(x=PC1, y=PC2)) +
  #    geom_point(color=K+1, shape=K, size=1, alpha = 0.5) +
  #    theme_classic() +
  #    labs(x="",y="")  
  #} else{
  ggplot(X[,1:2], aes(x=PC1, y=PC2,color=as.factor(K+1))) +
    geom_point(shape=19, size=1, show.legend=F) +
    theme_classic() +
    labs(x=xlab,y=ylab) +
    ggtitle(title) +
    theme(plot.title = element_text(face="bold", hjust = 0.5))
  #annotate("text", x=mu_x, y=mu_y, label=1:max(K), size=5, fontface = "bold")
  #}
}

draw.obs7 <- function(Y,K,title="",xlab="",ylab=""){
  
  X = data.frame(PC1=Y[,1],PC2=Y[,2])
  
  mu_x = sapply(1:max(K),function(z){mean(X[which(K==z),1])})
  mu_y = sapply(1:max(K),function(z){mean(X[which(K==z),2])})
  
  #if(title==""){
  #  ggplot(X[,1:2], aes(x=PC1, y=PC2)) +
  #    geom_point(color=K+1, shape=K, size=1, alpha = 0.5) +
  #    theme_classic() +
  #    labs(x="",y="")  
  #} else{
  ggplot(X[,1:2], aes(x=PC1, y=PC2,color=as.factor(K+1))) +
    geom_point(shape=19, size=1, show.legend=F) +
    theme_void() +
    theme(panel.border = element_rect(colour = "black", fill=NA, size=1))
  #annotate("text", x=mu_x, y=mu_y, label=1:max(K), size=5, fontface = "bold")
  #}
}

########################################################
# 10. Clustering methods
########################################################
library(flexclust) # for kcca
library(mclust) # for Mclust
library(class) # for knn
library(dbscan) # for dbscan

library('pcaReduce') # for pcaReduce
library("HMMVB") #for HMMVB
library("mclust") #for HMMVB
library("pryr") #for HMMVB

library("pcaReduce") #for pcareduce


kclst <- function(X, newdata){
  kcl = kcca(X, k=C, kccaFamily("kmeans"))
  res = predict(kcl, newdata=newdata)
  return(res)
}

mclst <- function(X, newdata){
  mcl = Mclust(X, G=G)
  res = predict(mcl, newdata=newdata)$classification
  return(res)
}

hclst <- function(X, newdata){
  hcl = hclust(dist(X))
  res = as.numeric(knn(X,newdata,cutree(hcl,C)))
  return(res)
}  

dclst <- function(X, newdata){
  dcl = dbscan(X, eps=E)
  res = predict(dcl, newdata=newdata, data=X)
  return(res)
}

pclst <- function(X, newdata){
  pcl <- PCAreduce(X, nbt=1, q=30, method='S')
  col.ind = which(apply(pcl[[1]],2,function(x) length(unique(x))) == C)
  res = as.numeric(knn(X,newdata,pcl[[1]][,col.ind]))
  return(res)
}

hmmvb <- function(X, newdat){
  wd=getwd()
  setwd('../_Hmmvb_package_v1.3.2')
  packagewd = getwd()
  
  HMM_1 = new("HMMVB", ndim = dim(X)[2], nseq = NVB, ndim_seq = DSQ, ncom_seq = CSQ,
              c_path = packagewd)
  estimate(HMM_1, data = X, diagonal_flag = TRUE)
  res = cluster(HMM_1, newdat)
  return(res)
}

hmmvb2 <- function(X, newdat){
  wd=getwd()
  setwd('../_Hmmvb_package_v1.3.2')
  packagewd = getwd()
  
  HMM_1 = new("HMMVB", ndim = dim(X)[2], nseq = NVB, ndim_seq = DSQ, ncom_seq = CSQ,
              c_path = packagewd)
  
  write_hyper_file(HMM_1)
  WriteData(HMM_1,X,"Train_data.txt")
  WriteData(HMM_1,newdat,"Test_data.txt")
  system(paste("./trainmaster -i Train_data.txt -m model_binary.dat -b hyperparam_HMMVB.dat -v", sep=""))
  system(paste("./testsync -i Test_data.txt -m model_binary.dat -o refcls.dat -t 0.3 -u -l 11", sep=""))
  res = c(as.matrix(read.table("refcls.dat", skip=1)))
  
  setwd(wd)
  
  return(res)
}
########################################################
# Simulate C clusters from GMM with one rare cluster
########################################################
library(MCMCpack) # for riwish
library(mvtnorm)
library(clusterGeneration)

#set.seed(1155)
#n =1000; # Sample size  
#p = 20; # dimension
#C = 8; # number of clusters
#k = 4; # number of small clusters

GenData <- function(n,p,C,k){
  #pp_true <- c(rep(0.025,k), rep((1-(0.025*k))/(C-k),C-k))   ## prob that cluster k is geneated
  pp = sort(runif(C-1,0,1))
  pp_true <- c(pp,1) - c(0,pp)   ## prob that cluster k is geneated
  mu_true = matrix(0, nrow = p, ncol =C)     ## centroid of each cluster
  #mu_true[,1] <- rep(5, p)
  #mu_true[,2] <- rep(-5,p)
  #mu_true[,3:C] <- matrix(runif(p*(C-2),-5,5), nrow = p, ncol = C-2)
  mu_true[,1:C] <- matrix(runif(p*C,-5,5), nrow = p, ncol = C)
  
  Sigma_true = array(0, dim = c(p,p,C))    ## var-cov matrix of x,y coordinates 
  #Sigma_true[,,1] <- diag(2,p)
  #Sigma_true[,,2] <- Sigma_true[,,1]
  #for(ii in 3:C){
  #  Sigma_true[,,ii] = riwish(p+13, diag(5, p))
  #}
  V = list()
  for(ii in 1:C){
    #Sigma_true[,,ii] = riwish(p+13, diag(5, p))
    V[[ii]] = genPositiveDefMat(dim=p)$Sigma
    Sigma_true[,,ii] = riwish(5*p, V[[ii]])
  }
  
  rsample <- rmultinom(n, 1, pp_true)      ## C by n, indicator of sample
  z_true <- apply(rsample, 2, function(x)which(x == 1))
  X <- t(sapply(z_true, function(x) {rmvnorm(1, mu_true[,x], Sigma_true[,,x])})) #nxp
  
  return(list(X=X,z=z_true))
}  
########################################################

GenData2 <- function(n,p,C,k){
  pp_true <- c(rep(0.025,k), rep((1-(0.025*k))/(C-k),C-k))   ## prob that cluster k is geneated
  mu_true = matrix(0, nrow = p, ncol =C)     ## centroid of each cluster
  #mu_true[,1] <- rep(5, p)
  #mu_true[,2] <- rep(-5,p)
  #mu_true[,3:C] <- matrix(runif(p*(C-2),-5,5), nrow = p, ncol = C-2)
  mu_true[,1:C] <- matrix(runif(p*C,-3,3), nrow = p, ncol = C)
  
  Sigma_true = array(0, dim = c(p,p,C))    ## var-cov matrix of x,y coordinates 
  #Sigma_true[,,1] <- diag(2,p)
  #Sigma_true[,,2] <- Sigma_true[,,1]
  #for(ii in 3:C){
  #  Sigma_true[,,ii] = riwish(p+13, diag(5, p))
  #}
  for(ii in 1:C){
    Sigma_true[,,ii] = riwish(3/2*p, genPositiveDefMat(dim=p)$Sigma)
  }
  
  rsample <- rmultinom(n, 1, pp_true)      ## C by n, indicator of sample
  z_true <- apply(rsample, 2, function(x)which(x == 1))
  X <- t(sapply(z_true, function(x) {rmvnorm(1, mu_true[,x], Sigma_true[,,x])})) #nxp
  
  return(list(X=X,z=z_true))
}  
########################################################



