#' Generates multi-dimensional random partition.
#'
#' This function generates multi-dimensional Gaussian random partition for simulation study.
#' @param n Size of observations.
#' @param p Dimension of data.
#' @param C The number of clusters.
#' @param k Default is 0. The number of small clusters. Proportion of sum of small clusters is set to have 10\% of data.
#' @param s Positive number. Default is 5. A partition is generated in (-s,s) p-cells.
#' @param dg Positive number. Default is 5*p. Degree of inverse Wishart distribution, which determine the separability of clusters.
#' @return List of n by p data coordinates matrix (X) and n length cluster label vector (m).
#' @examples
#' # set OTA package directory and compile C code.
#' SetOTA(directory)
#' set.seed(2070)
#' dat = GenData(n=5000,p=2,C=4)
#' # set parameters for 'kmeans' method in GenBsSamps.
#' C=4 
#' bs.samps = GenBsSamps(dat$X,100,"kmeans")
#' # find mean partition and uncertainty statistics.
#' idx = align2(bs.samps$Z,100)
#' res = MeanPart(bs.samps$Z, 100, idx)
#' # plot the result on two dimensional space.
#' plot.part(dat$X,dat$z)          # ground truth
#' plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
#' plot.part(dat$X,res$repre)      # OTA
#' # distance between ground truth and each partition
#' WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
#' @export


GenData <- function(n,p,C,k=0,s=5,dg=5){
  
  if(k==0){
    pp = sort(runif(C-1,0,1))
    pp_true <- c(pp,1) - c(0,pp)
  } else if(k>0 & k<C) {
    pp = 1/(k*10)
    pp_true <- c(rep(pp,k), rep((1-(pp*k))/(C-k),C-k))   ## prob that cluster k is geneated
  } else {
    warning("k is not appropriate.")
    break
  }
  
  mu_true = matrix(0, nrow = p, ncol = C)     ## centroid of each cluster
  mu_true[,1:C] <- matrix(runif(p*C,-s,s), nrow = p, ncol = C)
  
  
  Sigma_true = array(0, dim = c(p,p,C))    ## var-cov matrix of x,y coordinates 
  for(ii in 1:C){
    Sigma_true[,,ii] = MCMCpack::riwish(dg, clusterGeneration::genPositiveDefMat(dim=p)$Sigma)
  }
  
  rsample <- rmultinom(n, 1, pp_true)      ## C by n, indicator of sample
  z_true <- apply(rsample, 2, function(x)which(x == 1))
  X <- t(sapply(z_true, function(x) {mvtnorm::rmvnorm(1, mu_true[,x], Sigma_true[,,x])})) #nxp
  
  return(list(X=X,z=z_true))
}  