#' Generates perturbed samples by adding noise.
#'
#' This function generates ensemble of partitions by adding Gaussian noise.
#' @param X Original data matrix.
#' @param nb The number of ensemble partitions (perturbed samples) to be generated.
#' @param method Baseline clustering methods. User specified function or example methods included in package ("kmeans", "Mclust", "hclust", "dbscan", "PCAreduce", "HMM-VB") can be used. Refer to the Detail.
#' @param stack Default is TRUE. The format of the output ensemble partition. If it is TRUE, a stacked vector is returned. If not, a matrix is returned.
#' @keywords ensemble
#' @return List of ensemble of partitions and detected number of clusters for each partition.
#' @examples
#' # set OTA package directory and compile C code.
#' SetOTA(directory)
#' set.seed(2070)
#' dat = GenData(n=5000,p=2,C=4)
#' # set parameters for 'kmeans' method in GenBsSamps.
#' C=4 
#' bs.samps = GenBsSamps(dat$X,100,"kmeans")
#' # find mean partition and uncertainty statistics.
#' idx = align2(bs.samps$Z,100)
#' res = MeanPart(bs.samps$Z, 100, idx)
#' # plot the result on two dimensional space.
#' plot.part(dat$X,dat$z)          # ground truth
#' plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
#' plot.part(dat$X,res$repre)      # OTA
#' # distance between ground truth and each partition
#' WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
#' @export

GenNoiseSamps <- function(X, nb, method, stack=T, pair.dist=NULL){

  if(is.function(method)){
    clustering = method
  } else if(is.character(method)){
    method = switch(method, 
                    "kmeans" = "kclst",
                    "Mclust" = "mclst",
                    "hclust" = "hclst",
                    "dbscan" = "dclst",
                    "PCAreduce" = "pclst",
                    "HMM-VB" = "hmmvb2")
    if(is.null(method)){
      warning("Choose a method from 'kmeans', 'Mclust', 'hclust', 'dbscan', 'PCAreduce', 'HMM-VB', or specify a function.")
      break
    } else {
      clustering = eval(parse(text=method))
    }
  } else {
    warning("Choose a method from 'kmeans', 'Mclust', 'hclust', 'dbscan', 'PCAreduce', 'HMM-VB', or specify a function.")
    break
  }
  
  #--------------------------------------------------
  
  n <- nrow(X)                      #n: number of observations
  bet <- array(0, dim = c(nb, n))   #nb: number of bootstrap samples
  Zb <- array(0, dim = c(nb,n))     
  
  dim <- ncol(X)
  if(is.null(pair.dist)){
    pair.dist = matrix(NA,n,n)
    for(i in 1:n){
      for(j in 1:i){
        pair.dist[i,j] = sqrt(sum((X[i,]-X[j,])^2))
      }
    }
  }
  
  sig = quantile(pair.dist, probs=0.1, na.rm=T)
  
  Zb.stack <- NULL
  m <- rep(NA,nb)
  for (i in 1:nb){
    noise = mvtnorm::rmvnorm(n,mean=rep(0,dim),sigma=sig*diag(dim)) # generate random noises
    Xb = as.data.frame(X+noise)
    
    Zb[i,] = clustering(Xb, newdata=X)      # train with bootstraped sample
    # and predict using new cluster
    m[i] = length(unique(Zb[i,]))
    Zb.stack = c(Zb.stack, Zb[i,])
    print(i)
  }
  
  if(stack==T){
    return(list(Z=Zb.stack,m=m))
  } else {
    return(list(Z=Zb,m=m))
  }
}
