#' Align ensemble of partitions to the mean partition.
#'
#' This function aligns ensemble of partitions to the reference partition and return mean partition and its stability measures.
#' @param Zb.stack Stacked cluster label vector (n times nb length).
#' @param nb The number of ensemble partitions in Zb.stack.
#' @param idx Index of reference partition in ensemble. (idx must be in [1,nb])
#' @param threshold Default is 0.8. Threshold for matching statistic. This percentage of overlapping between two clusters is regarded as matching.
#' @param alpha Default is 0.1. Confident level for confidence point set. 
#' @return List of mean partition(repre), distance of mean partition to each partition in ensemble(dist), output(res), weight matrix(wt), topological stability statistics(summary), confidence point set(confset), cluster alignment and point-based(CAP) separability(jaccard_mat).
#' @examples
#' # set OTA package directory and compile C code.
#' SetOTA(directory)
#' set.seed(2070)
#' dat = GenData(n=5000,p=2,C=4)
#' # set parameters for 'kmeans' method in GenBsSamps.
#' C=4 
#' bs.samps = GenBsSamps(dat$X,100,"kmeans")
#' # find mean partition and uncertainty statistics.
#' idx = align2(bs.samps$Z,100)
#' res = MeanPart(bs.samps$Z, 100, idx)
#' # plot the result on two dimensional space.
#' plot.part(dat$X,dat$z)          # ground truth
#' plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
#' plot.part(dat$X,res$repre)      # OTA
#' # distance between ground truth and each partition
#' WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
#' @export


MeanPart <- function(Zb.stack, nb, idx, threshold=0.8, alpha=0.1){
  n = length(Zb.stack)/nb
  K.rf = max(Zb.stack[(n*(idx-1)+1):(n*idx)])
  
  pwd <- getwd()
  setwd(directory)
  
  rfcls = Zb.stack[(n*(idx-1)+1):(n*idx)]
  zbcls = c(rfcls,Zb.stack)-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  system(paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb+1," -t ",threshold," -a ",alpha," -2",sep="")
         ,ignore.stdout = T)
  
  dist <- read.table("zb.par")
  wt <- read.table("zb.wt")
  
  setwd(pwd)
  
  m = dist[-1,1]
  
  P.raw = list()
  
  for(i in 1:nb){
    P.raw[[i]] = matrix(0,n,m[i])
    for(k in 1:n){
      P.raw[[i]][k, Zb.stack[n*(i-1)+k]] = 1
    }
  }  
  
  wt = as.matrix(wt)
  
  P.tild = matrix(NA,n,K.rf*nb)
  P.tild.sum = matrix(0,n,K.rf)
  
  for(k in 1:nb){
    wtsub = wt[(sum(m[0:(k-1)])+1):sum(m[0:k]),]
    P.tild = P.raw[[k]]%*%(wtsub/ifelse(rowSums(wtsub)==0,1,rowSums(wtsub)))
    P.tild.sum = P.tild.sum + P.tild
  }
  
  P.bar = P.tild.sum/nb
  P.bar.hrd.asgn = apply(P.bar,1,which.max)
  K.rf = max(P.bar.hrd.asgn)
  
  setwd(directory)
  
  rfcls = P.bar.hrd.asgn
  zbcls = c(rfcls,Zb.stack)-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  system(paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb+1," -t ",threshold," -a ",alpha," -2",sep=""))
  
  dist <- read.table("zb.par")
  res <- read.table("zb.ls") 
  wt <- read.table("zb.wt")
  smry <- read.table("zb.summary",nrows=K.rf)
  jaccard_mat <- read.table("zb.summary",skip=K.rf,nrows=K.rf)
  confset.idx <- list()
  confset <- list()
  for(i in 1:K.rf){
    confset.idx[[i]] <- as.integer(read.table("zb.summary",skip=(2*K.rf+i-1),nrows=1))[-c(1,2)]+1
    confset[[i]] = rep(0,n)
    confset[[i]][confset.idx[[i]]]=1
    
  }
  
  setwd(pwd)
  
  return(list(repre=P.bar.hrd.asgn,
              dist=dist,res=res,wt=wt,
              summary=smry,
              jaccard_mat = jaccard_mat,
              confset=confset))
}