#' Wasserstein distance between partitions.
#'
#' This function calculate Wasserstein distance between reference partition and ensemble of partitions.
#' @param Zb.stack Stacked cluster label vector. Reference cluster should be on top of ensemble or partitions. Wasserstein distances of ensemble of partitions to the reference partition is returned.
#' @param nb The number of ensemble partitions in Zb.stack.
#' @return Matrix of number of clusters and Wasserstein distances of ensemble of partitions to the reference partition.
#' @examples
#' # set OTA package directory and compile C code.
#' SetOTA("/Users/BSS/Documents/7.Research/Cluster_Stability_2/r-package/package5")
#' set.seed(2070)
#' dat = GenData(n=5000,p=2,C=4)
#' # set parameters for 'kmeans' method in GenBsSamps.
#' C=4 
#' bs.samps = GenBsSamps(dat$X,100,"kmeans")
#' # find mean partition and uncertainty statistics.
#' idx = align2(bs.samps$Z,100)
#' res = MeanPart(bs.samps$Z, 100, idx)
#' # plot the result on two dimensional space.
#' plot.part(dat$X,dat$z)          # ground truth
#' plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
#' plot.part(dat$X,res$repre)      # OTA
#' # distance between ground truth and each partition
#' WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
#' @export


WassDist <- function(Zb.stack, nb){
  threshold = 0.8; alpha = 0.1;
  
  n = length(Zb.stack)/nb
  
  pwd <- getwd()
  setwd(directory)
  
  zbcls = c(Zb.stack)-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  system(paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb," -t ",threshold," -a ",alpha," -2",sep="")
         ,ignore.stdout = T)
  
  
  dist <- read.table("zb.par")

  setwd(pwd)
  
  return(dist)
}