#' Align partitions to the reference partition.
#'
#' This function aligns partitions to the reference partition by optimal transport alignment(OTA).
#' @param Zb.stack Stacked cluster label vector. Reference cluster should be on top of ensemble or partitions.
#' @param nb The number of ensemble partitions in Zb.stack.
#' @param threshold Default is 0.8. Threshold for matching statistic. This percentage of overlapping between two clusters is regarded as matching.
#' @param alpha Default is 0.1. Confident level for confidence point set. 
#' @param split Logical. If it is TRUE, split is counted as match in confidence set calculation.
#' @param folder Default is "align". The folder name where temporary result files are created.
#' @return List of mean partition(repre), distance of mean partition to each partition in ensemble(dist), output(res), weight matrix(wt), topological stability statistics(summary), confidence point set(confset), cluster alignment and point-based(CAP) separability(jaccard_mat).
#' @examples
#' # set OTA package directory and compile C code.
#' SetOTA(directory)
#' set.seed(2070)
#' dat = GenData(n=5000,p=2,C=4)
#' # set parameters for 'kmeans' method in GenBsSamps.
#' C=4 
#' bs.samps = GenBsSamps(dat$X,100,"kmeans")
#' # find mean partition and uncertainty statistics.
#' idx = align2(bs.samps$Z,100)
#' res = MeanPart(bs.samps$Z, 100, idx)
#' # plot the result on two dimensional space.
#' plot.part(dat$X,dat$z)          # ground truth
#' plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
#' plot.part(dat$X,res$repre)      # OTA
#' # distance between ground truth and each partition
#' WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
#' @export


align.control <- function(Zb.stack, nb, threshold=0.8, alpha=0.1, split=F, folder="align"){
  pwd <- getwd()
  setwd(directory)
  
  zbcls = Zb.stack-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  comnd = paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb+1," -t ",threshold," -a ",alpha," -2",sep="")
  if(split==T){
    comnd = paste(comnd," -s",sep="")
  }
  system(comnd)
  
  dist <- read.table("zb.par")
  res <- read.table("zb.ls") 
  wt <- read.table("zb.wt")
  smry <- read.table("zb.summary",nrows=K.rf)
  jaccard_mat <- read.table("zb.summary",skip=K.rf,nrows=K.rf)
  confset.idx <- list()
  confset <- list()
  for(i in 1:K.rf){
    confset.idx[[i]] <- as.integer(read.table("zb.summary",skip=(2*K.rf+i-1),nrows=1))[-c(1,2)]+1
    confset[[i]] = rep(0,n)
    confset[[i]][confset.idx[[i]]]=1
    
  }
  
  setwd(pwd)
  
  return(list(repre=P.bar.hrd.asgn,
              dist=dist,res=res,wt=wt,
              summary=smry,
              jaccard_mat = jaccard_mat,
              confset=confset))
  
}