#' Median partition by Optimal Transport Alignment(OTA)
#'
#' This function searches the index of the median partition in ensemble of partitions by optimal transport alignment(OTA).
#' @param Zb.stack Stacked cluster label vector (n times nb length).
#' @param nb The number of ensemble partitions in Zb.stack.
#' @param folder Default is "align". The folder name where temporary result files are created.
#' @return Median partition index in ensemble.
#' @examples
#' # set OTA package directory and compile C code.
#' SetOTA(directory)
#' set.seed(2070)
#' dat = GenData(n=5000,p=2,C=4)
#' # set parameters for 'kmeans' method in GenBsSamps.
#' C=4 
#' bs.samps = GenBsSamps(dat$X,100,"kmeans")
#' # find mean partition and uncertainty statistics.
#' idx = align2(bs.samps$Z,100)
#' res = MeanPart(bs.samps$Z, 100, idx)
#' # plot the result on two dimensional space.
#' plot.part(dat$X,dat$z)          # ground truth
#' plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
#' plot.part(dat$X,res$repre)      # OTA
#' # distance between ground truth and each partition
#' WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
#' @export


align2 <- function(Zb.stack, nb, folder="align"){
  pwd <- getwd()
  setwd(directory)
  if(!(dir.exists(folder))) dir.create(folder)
  file.copy("labelsw_bunch2", folder)
  setwd(folder)
  
  n = length(Zb.stack)/nb
  rfcls = rep(1,n)
  zbcls = c(rfcls,Zb.stack)-1
  write(zbcls, file = "zb.cls", ncolumns = 1)
  idx = as.numeric(system(paste("./labelsw_bunch2 -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb+1," -2",sep=""), intern=T))
  
  setwd(pwd)
  
  return(idx)
  
}